/**
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 * 
 * Based on code from notification.h in libnotify.
 * Copyright (C) 2006 Christian Hammond.
 * Copyright (C) 2006 John Palmieri.
 * Copyright (C) 2010 Red Hat, Inc.
 * 
 * This software is published by the license of CPU-OS License，you can use and 
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us 
 * by email <support_os@cpu-os.ac.cn>  
 * 
**/

/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
 *
 * Copyright (C) 2006 Christian Hammond
 * Copyright (C) 2006 John Palmieri
 * Copyright (C) 2010 Red Hat, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

#ifndef _CDOS_NOTIFY_NOTIFICATION_H_
#define _CDOS_NOTIFY_NOTIFICATION_H_

#include <glib.h>
#include <glib-object.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

/**
 * SECTION:Cdos-API
 * @short_description: A passive pop-up notification.
 * @title: CdosNotifyNotification
 *
 * #CdosNotifyNotification represents a passive pop-up notification. It can
 * contain summary text, body text, and an icon, as well as hints specifying
 * how the notification should be presented. The notification is rendered
 * by a notification daemon, and may present the notification in any number
 * of ways. As such, there is a clear separation of content and presentation,
 * and this API enforces that.
 */

G_BEGIN_DECLS

/**
 * CDOS_NOTIFY_EXPIRES_DEFAULT:
 *
 * The default expiration time on a notification.
 */
#define CDOS_NOTIFY_EXPIRES_DEFAULT -1

/**
 * CDOS_NOTIFY_EXPIRES_NEVER:
 *
 * The notification never expires. It stays open until closed by the calling API
 * or the user.
 */
#define CDOS_NOTIFY_EXPIRES_NEVER    0

#define CDOS_NOTIFY_TYPE_NOTIFICATION         (cdos_notify_notification_get_type ())
#define CDOS_NOTIFY_NOTIFICATION(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), CDOS_NOTIFY_TYPE_NOTIFICATION, CdosNotifyNotification))
#define CDOS_NOTIFY_NOTIFICATION_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST((k), CDOS_NOTIFY_TYPE_NOTIFICATION, CdosNotifyNotificationClass))
#define NOTIFY_IS_CDOS_NOTIFICATION(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), CDOS_NOTIFY_TYPE_NOTIFICATION))
#define NOTIFY_IS_CDOS_NOTIFICATION_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), CDOS_NOTIFY_TYPE_NOTIFICATION))
#define CDOS_NOTIFY_NOTIFICATION_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), CDOS_NOTIFY_TYPE_NOTIFICATION, CdosNotifyNotificationClass))

typedef struct _CdosNotifyNotification CdosNotifyNotification;
typedef struct _CdosNotifyNotificationClass CdosNotifyNotificationClass;
typedef struct _CdosNotifyNotificationPrivate CdosNotifyNotificationPrivate;

struct _CdosNotifyNotification
{
        /*< private >*/
        GObject                    parent_object;

        CdosNotifyNotificationPrivate *priv;
};

struct _CdosNotifyNotificationClass
{
        GObjectClass    parent_class;

        /* Signals */
        void            (*closed) (CdosNotifyNotification *notification);
};


/**
 * CdosNotifyUrgency:
 * @CDOS_NOTIFY_URGENCY_LOW: Low urgency. Used for unimportant notifications.
 * @CDOS_NOTIFY_URGENCY_NORMAL: Normal urgency. Used for most standard notifications.
 * @CDOS_NOTIFY_URGENCY_CRITICAL: Critical urgency. Used for very important notifications.
 *
 * The urgency level of the notification.
 */
typedef enum
{
        CDOS_NOTIFY_URGENCY_LOW,
        CDOS_NOTIFY_URGENCY_NORMAL,
        CDOS_NOTIFY_URGENCY_CRITICAL,

} CdosNotifyUrgency;

/**
 * CdosNotifyActionCallback:
 * @notification: The notification.
 * @action: The action.
 * @user_data: User data.
 *
 * An action callback function.
 */
typedef void    (*CdosNotifyActionCallback) (CdosNotifyNotification *notification,
                                         char               *action,
                                         gpointer            user_data);

/**
 * CDOS_NOTIFY_ACTION_CALLBACK:
 * @func: The function to cast.
 *
 * A convenience macro for casting a function to a #CdosNotifyActionCallback. This
 * is much like G_CALLBACK().
 */
#define CDOS_NOTIFY_ACTION_CALLBACK(func) ((CdosNotifyActionCallback)(func))

GType               cdos_notify_notification_get_type             (void);

/**
 * cdos_notify_notification_new:
 * @summary: The required summary text.
 * @body: (allow-none): The optional body text.
 * @icon: (allow-none): The optional icon theme icon name or filename.
 *
 * Creates a new #CdosNotifyNotification. The summary text is required, but
 * all other parameters are optional.
 *
 * Returns: The new #CdosNotifyNotification.
 */
CdosNotifyNotification *cdos_notify_notification_new                  (const char         *summary,
                                                              const char         *body,
                                                              const char         *icon);

/**
 * cdos_notify_notification_update:
 * @notification: The notification to update.
 * @summary: The new required summary text.
 * @body: (allow-none): The optional body text.
 * @icon: (allow-none): The optional icon theme icon name or filename.
 *
 * Updates the notification text and icon. This won't send the update out
 * and display it on the screen. For that, you will need to call
 * cdos_notify_notification_show().
 *
 * Returns: %TRUE, unless an invalid parameter was passed.
 */
gboolean            cdos_notify_notification_update                (CdosNotifyNotification *notification,
                                                               const char         *summary,
                                                               const char         *body,
                                                               const char         *icon);

/**
 * cdos_notify_notification_show:
 * @notification: The notification.
 * @error: The returned error information.
 *
 * Tells the notification server to display the notification on the screen.
 *
 * Returns: %TRUE if successful. On error, this will return %FALSE and set
 *          @error.
 */
gboolean            cdos_notify_notification_show                  (CdosNotifyNotification *notification,
                                                               GError            **error);

/**
 * cdos_notify_notification_set_timeout:
 * @notification: The notification.
 * @timeout: The timeout in milliseconds.
 *
 * Sets the timeout of the notification. To set the default time, pass
 * %CDOS_NOTIFY_EXPIRES_DEFAULT as @timeout. To set the notification to never
 * expire, pass %CDOS_NOTIFY_EXPIRES_NEVER.
 *
 * Note that the timeout may be ignored by the server.
 */
void                cdos_notify_notification_set_timeout           (CdosNotifyNotification *notification,
                                                               gint                timeout);

/**
 * cdos_notify_notification_set_category:
 * @notification: The notification.
 * @category: The category.
 *
 * Sets the category of this notification. This can be used by the
 * notification server to filter or display the data in a certain way.
 */
void                cdos_notify_notification_set_category          (CdosNotifyNotification *notification,
                                                               const char         *category);

/**
 * cdos_notify_notification_set_urgency:
 * @notification: The notification.
 * @urgency: The urgency level.
 *
 * Sets the urgency level of this notification.
 *
 * See: #CdosNotifyUrgency
 */
void                cdos_notify_notification_set_urgency           (CdosNotifyNotification *notification,
                                                               CdosNotifyUrgency       urgency);

/**
 * cdos_notify_notification_set_image_from_pixbuf:
 * @notification: The notification.
 * @pixbuf: The image.
 *
 * Sets the image in the notification from a #GdkPixbuf.
 *
 */
void                cdos_notify_notification_set_image_from_pixbuf (CdosNotifyNotification *notification,
                                                               GdkPixbuf          *pixbuf);

#ifndef LIBNOTIFY_DISABLE_DEPRECATED

/**
 * cdos_notify_notification_set_icon_from_pixbuf:
 * @notification: The notification.
 * @icon: The icon.
 *
 * Sets the icon in the notification from a #GdkPixbuf.
 * Deprecated: use cdos_notify_notification_set_image_from_pixbuf() instead.
 *
 */
void                cdos_notify_notification_set_icon_from_pixbuf  (CdosNotifyNotification *notification,
                                                               GdkPixbuf          *icon);

/**
 * cdos_notify_notification_set_hint_int32:
 * @notification: The notification.
 * @key: The hint.
 * @value: The hint's value.
 *
 * Sets a hint with a 32-bit integer value.
 *
 * Deprecated: 0.6. Use cdos_notify_notification_set_hint() instead
 */
void                cdos_notify_notification_set_hint_int32        (CdosNotifyNotification *notification,
                                                               const char         *key,
                                                               gint                value);

/**
 * cdos_notify_notification_set_hint_uint32:
 * @notification: The notification.
 * @key: The hint.
 * @value: The hint's value.
 *
 * Sets a hint with an unsigned 32-bit integer value.
 *
 * Deprecated: 0.6. Use cdos_notify_notification_set_hint() instead
 */
void                cdos_notify_notification_set_hint_uint32       (CdosNotifyNotification *notification,
                                                               const char         *key,
                                                               guint               value);

/**
 * cdos_notify_notification_set_hint_double:
 * @notification: The notification.
 * @key: The hint.
 * @value: The hint's value.
 *
 * Sets a hint with a double value.
 *
 * Deprecated: 0.6. Use cdos_notify_notification_set_hint() instead
 */
void                cdos_notify_notification_set_hint_double       (CdosNotifyNotification *notification,
                                                               const char         *key,
                                                               gdouble             value);

/**
 * cdos_notify_notification_set_hint_string:
 * @notification: The notification.
 * @key: The hint.
 * @value: The hint's value.
 *
 * Sets a hint with a string value.
 *
 * Deprecated: 0.6. Use cdos_notify_notification_set_hint() instead
 */
void                cdos_notify_notification_set_hint_string       (CdosNotifyNotification *notification,
                                                               const char         *key,
                                                               const char         *value);

/**
 * cdos_notify_notification_set_hint_byte:
 * @notification: The notification.
 * @key: The hint.
 * @value: The hint's value.
 *
 * Sets a hint with a byte value.
 *
 * Deprecated: 0.6. Use cdos_notify_notification_set_hint() instead
 */
void                cdos_notify_notification_set_hint_byte         (CdosNotifyNotification *notification,
                                                               const char         *key,
                                                               guchar              value);

/**
 * cdos_notify_notification_set_hint_byte_array:
 * @notification: The notification.
 * @key: The hint.
 * @value: (array length=len): The hint's value.
 * @len: The length of the byte array.
 *
 * Sets a hint with a byte array value. The length of @value must be passed
 * as @len.
 *
 * Deprecated: 0.6. Use cdos_notify_notification_set_hint() instead
 */
void                cdos_notify_notification_set_hint_byte_array   (CdosNotifyNotification *notification,
                                                               const char         *key,
                                                               const guchar       *value,
                                                               gsize               len);
#endif

/**
 * cdos_notify_notification_set_hint:
 * @notification: a #CdosNotifyNotification
 * @key: the hint key
 * @value: (allow-none): the hint value, or %NULL to unset the hint
 *
 * Sets a hint for @key with value @value. If @value is %NULL,
 * a previously set hint for @key is unset.
 *
 * If @value is floating, it is consumed.
 *
 * Since: 0.6
 */
void                cdos_notify_notification_set_hint              (CdosNotifyNotification *notification,
                                                               const char         *key,
                                                               GVariant           *value);

/**
 * cdos_notify_notification_set_app_name:
 * @notification: a #CdosNotifyNotification
 * @app_name: the localised application name
 *
 * Sets the application name for the notification. If this function is
 * not called or if @app_name is %NULL, the application name will be
 * set from the value used in cdos_notify_init() or overridden with
 * cdos_notify_set_app_name().
 *
 * Since: 0.7.3
 */
void                cdos_notify_notification_set_app_name          (CdosNotifyNotification *notification,
                                                               const char         *app_name);

/**
 * cdos_notify_notification_clear_hints:
 * @notification: The notification.
 *
 * Clears all hints from the notification.
 */
void                cdos_notify_notification_clear_hints           (CdosNotifyNotification *notification);

/**
 * cdos_notify_notification_add_action:
 * @notification: The notification.
 * @action: The action ID.
 * @label: The human-readable action label.
 * @callback: The action's callback function.
 * @user_data: Optional custom data to pass to @callback.
 * @free_func: (type GLib.DestroyNotify): An optional function to free @user_data when the notification
 *             is destroyed.
 *
 * Adds an action to a notification. When the action is invoked, the
 * specified callback function will be called, along with the value passed
 * to @user_data.
 */
void                cdos_notify_notification_add_action            (CdosNotifyNotification *notification,
                                                               const char         *action,
                                                               const char         *label,
                                                               CdosNotifyActionCallback callback,
                                                               gpointer            user_data,
                                                               GFreeFunc           free_func);

/**
 * cdos_notify_notification_clear_actions:
 * @notification: The notification.
 *
 * Clears all actions from the notification.
 */
void                cdos_notify_notification_clear_actions         (CdosNotifyNotification *notification);

/**
 * cdos_notify_notification_close:
 * @notification: The notification.
 * @error: The returned error information.
 *
 * Synchronously tells the notification server to hide the notification on the screen.
 *
 * Returns: %TRUE on success, or %FALSE on error with @error filled in
 */
gboolean            cdos_notify_notification_close                 (CdosNotifyNotification *notification,
                                                               GError            **error);

/**
 * cdos_notify_notification_get_closed_reason:
 * @notification: The notification.
 *
 * Returns the closed reason code for the notification. This is valid only
 * after the "closed" signal is emitted.
 *
 * Returns: The closed reason code.
 */
gint                cdos_notify_notification_get_closed_reason     (const CdosNotifyNotification *notification);

G_END_DECLS
#endif /* CDOS_NOTIFY_NOTIFICATION_H */
